@@ -55,7 +55,7 @@ copy_xattr (const char *from, const char *to)
     {
       if (errno != ENOSYS && errno != ENOTSUP)
 	{
-	  fprintf (stderr, _("listing attributes of %s\n"), from);
+	  fprintf (stderr, _("Cannot get attributes for `%s': %m\n"), from);
 	  return -1;
 	}
     }
@@ -73,7 +73,7 @@ copy_xattr (const char *from, const char *to)
       size = llistxattr (from, names, size);
       if (size < 0)
 	{
-	  fprintf (stderr, _("listing attributes of %s\n"), from);
+	  fprintf (stderr, _("Cannot get attributes for `%s': %m\n"), from);
 	  if (value)
 	    free (value);
 	  free (names);
@@ -97,7 +97,7 @@ copy_xattr (const char *from, const char *to)
 	  size = lgetxattr (from, name, NULL, 0);
 	  if (size < 0)
 	    {
-	      fprintf (stderr, _("getting attribute %s of %s\n"),
+	      fprintf (stderr, _("Cannot get attribute %s of `%s': %m\n"),
 		       name, from);
 	      if (value)
 		free (value);
@@ -117,7 +117,7 @@ copy_xattr (const char *from, const char *to)
 	  size = lgetxattr (from, name, value, size);
 	  if (size < 0)
 	    {
-	      fprintf (stderr, _("getting attribute %s of %s\n"),
+	      fprintf (stderr, _("Cannot get attribute %s of `%s': %m\n"),
 		       name, from);
 	      if (value)
 		free (value);
@@ -127,10 +127,13 @@ copy_xattr (const char *from, const char *to)
 
 	  if (lsetxattr (to, name, value, size, 0) != 0)
 	    {
-	      if (errno == ENOSYS)
-		fprintf (stderr, _("setting attributes for %s\n"), to);
+	      if (strcmp (name, "security.selinux") == 0 &&
+		  is_selinux_enabled() == 0)
+		fprintf (stderr,
+			 _("SELinux not enabled, ignore attribute %s for `%s'.\n"),
+			 name, to);
 	      else
-		fprintf (stderr, _("setting attribute %s for %s\n"),
+		fprintf (stderr, _("Cannot set attribute %s for `%s': %m\n"),
 			 name, to);
 	      if (value)
 		free (value);
