@@ -205,7 +205,9 @@ change_shadow_info (user_t *data)
   free (buf);
   if (res == NULL)
     return E_FAILURE;
-  else if (strcmp (res, "1969-12-31") == 0)
+  else if (strcmp (res, "1969-12-31") == 0 ||
+	   strcmp (res, "0") == 0 ||
+	   strcmp (res, "-1") == 0)
     data->sp.sp_lstchg = -1;
   else
     {
@@ -223,7 +225,9 @@ change_shadow_info (user_t *data)
   free (buf);
   if (res == NULL)
     return E_FAILURE;
-  else if (strcmp (res, "1969-12-31") == 0)
+  else if (strcmp (res, "1969-12-31") == 0 ||
+	   strcmp (res, "0") == 0 ||
+	   strcmp (res, "-1") == 0)
     data->spn.sp_expire = -1;
   else
     {
@@ -495,30 +499,8 @@ main (int argc, char *argv[])
     }
 
 #ifdef USE_LDAP
-  if (binddn && pw_data->service == S_LDAP && !l_flag)
-    {
-      /* A user tries to change data stored in a LDAP database and
-	 knows the Manager dn, now we need the password from him.  */
-      ldap_session_t *session = create_ldap_session (LDAP_PATH_CONF);
-      char *cp;
-
-      if (session == NULL)
-	return E_FAILURE;
-
-      cp = getpass (_("Enter LDAP Password:"));
-
-      pw_data->binddn = strdup (binddn);
-
-      if (open_ldap_session (session) != 0)
-	return E_FAILURE;
-
-      if (ldap_authentication (session, NULL, binddn, cp) != 0)
-	return E_NOPERM;
-
-      close_ldap_session (session);
-
-      pw_data->oldclearpwd = strdup (cp);
-    }
+  if (binddn && pw_data->service == S_LDAP)
+    pw_data->oldclearpwd = strdup (get_ldap_password (binddn));
   else
 #endif /* USE_LDAP */
     if (do_authentication (program, caller_name, pw_data) != 0)
@@ -531,7 +513,8 @@ main (int argc, char *argv[])
 
   if (l_flag)
     {
-      if (uid != 0 && strcmp (caller_name, pw_data->pw.pw_name) != 0)
+      if (uid != 0 && pw_data->service != S_LDAP &&
+	  strcmp (caller_name, pw_data->pw.pw_name) != 0)
 	{
 	  fprintf (stderr,
 		   _("You can only list your own aging information.\n"));
