@@ -22,9 +22,11 @@
 #define _GNU_SOURCE
 
 #include <pwd.h>
+#include <wchar.h>
 #include <ctype.h>
 #include <errno.h>
 #include <stdio.h>
+#include <wctype.h>
 #include <signal.h>
 #include <unistd.h>
 #include <getopt.h>
@@ -219,22 +221,42 @@ parse_passwd (const char *gecos, struct fn_info *info)
     }
 }
 
+/* convert a multibye string to a wide character string, so
+   that we can use iswprint.  */
+static wchar_t *
+mbstowcs_alloc (const char *string)
+{
+  size_t size = strlen (string) + 1;
+  wchar_t *buf = malloc (size * sizeof (wchar_t));
+
+  if (buf == NULL)
+    return NULL;
+
+  size = mbstowcs (buf, string, size);
+  if (size == (size_t) -1)
+    return NULL;
+  buf = realloc (buf, (size + 1) * sizeof (wchar_t));
+  return buf;
+}
+
 static int
 check_field (const char *program, const char *field, const char *illegal)
 {
+  wchar_t *wstr = mbstowcs_alloc (field);
+  wchar_t *willegal = mbstowcs_alloc (illegal);
   size_t i;
-  int c;
 
   /* keep /etc/passwd clean. */
-  for (i = 0; i < strlen (field); i++)
+  for (i = 0; i < wcslen (wstr); i++)
     {
-      c = field[i];
-      if (strchr (illegal, c) != NULL || c == '"' || c == '\n')
+      wchar_t c = wstr[i];
+      if (wcschr (willegal, c) != NULL || c == '"' || c == '\n')
 	{
-	  printf (_("%s: '%c' is not allowed.\n"), program, c);
+	  printf (_("%s: The characters '%s\"' are not allowed.\n"),
+		  program, illegal);
 	  return 1;
         }
-      if (iscntrl (c))
+      if (iswcntrl (c))
 	{
 	  printf (_("%s: Control characters are not allowed.\n"), program);
 	  return 1;
@@ -608,29 +630,7 @@ main (int argc, char *argv[])
     {
 #ifdef USE_LDAP
       if (binddn && pw_data->service == S_LDAP)
-	{
-	  /* A user tries to change data stored in a LDAP database and
-	     knows the Manager dn, now we need the password from him.  */
-	  ldap_session_t *session = create_ldap_session (LDAP_PATH_CONF);
-	  char *cp;
-
-	  if (session == NULL)
-	    return E_FAILURE;
-
-	  cp = getpass (_("Enter LDAP Password:"));
-
-	  pw_data->binddn = strdup (binddn);
-
-	  if (open_ldap_session (session) != 0)
-	    return E_FAILURE;
-
-	  if (ldap_authentication (session, NULL, binddn, cp) != 0)
-	    return E_NOPERM;
-
-	  close_ldap_session (session);
-
-	  pw_data->oldclearpwd = strdup (cp);
-	}
+	pw_data->oldclearpwd = strdup (get_ldap_password (binddn));
       else
 #endif /* USE_LDAP */
 	if (do_authentication (program, caller_name, pw_data) != 0)
