@@ -156,32 +156,6 @@ is_primary_group (gid_t gid, int have_extrapath)
   return retval;
 }
 
-#ifdef USE_LDAP
-static char *
-get_ldap_password (const char *binddn)
-{
-  /* A user tries to add an account stored in a LDAP database and
-     knows the Manager dn, now we need the password from him.  */
-  ldap_session_t *session = create_ldap_session (LDAP_PATH_CONF);
-  char *cp;
-
-  if (session == NULL)
-    return NULL;
-
-  cp = getpass (_("Enter LDAP Password:"));
-
-  if (open_ldap_session (session) != 0)
-    return NULL;
-
-  if (ldap_authentication (session, NULL, binddn, cp) != 0)
-    return NULL;
-
-  close_ldap_session (session);
-
-  return cp;
-}
-#endif
-
 int
 main (int argc, char **argv)
 {
@@ -327,14 +301,34 @@ main (int argc, char **argv)
     }
 
 #ifdef USE_LDAP
-  if (binddn)
+  if (gr_data->service == S_LDAP)
     {
-      gr_data->binddn = strdup (binddn);
-      if (gr_data->service == S_LDAP)
+      if (binddn == NULL)
 	{
-	  char *cp = get_ldap_password (binddn);
+	  binddn = get_caller_dn ();
+	  if (binddn == NULL)
+	    {
+	      fprintf (stderr, _("%s: Cannot delete group from LDAP database without DN.\n"),
+		       program);
+	    }
+	  else gr_data->binddn = strdup (binddn);
+	}
+      else
+	gr_data->binddn = strdup (binddn);
+
+      if (gr_data->oldclearpwd == NULL)
+	{
+	  char *cp = get_ldap_password (gr_data->binddn);
+
 	  if (cp)
 	    gr_data->oldclearpwd = strdup (cp);
+	  else
+	    {
+	      fprintf (stderr,
+		       _("%s: Group not deleted from LDAP database.\n"),
+		       program);
+	      return E_FAILURE;
+	    }
 	}
     }
 #endif
