@@ -42,6 +42,7 @@
 #define E_BAD_ENTRY 2
 #define E_NO_FILE 3
 #define E_PWDBUSY 4
+#define E_FAILURE 5
 
 #ifndef _
 #define _(String) gettext (String)
@@ -70,7 +71,7 @@ print_help (const char *program)
   fputs (_("  -q, --quiet    Don't print warnings, only errors\n"), stdout);
   fputs (_("  -r, --read-only Run in read-only mode, don't make changes\n"),
 	 stdout);
-  fputs (_("  -s, --sort     Sort the group file, not checks done\n"),
+  fputs (_("  -s, --sort     Sort the group file, no checks are done\n"),
 	 stdout);
   fputs (_("      --help     Give this help list\n"), stdout);
   fputs (_("  -u, --usage    Give a short usage message\n"), stdout);
@@ -443,6 +444,232 @@ loop_over_group_file (int quiet)
   return result;
 }
 
+/* XXX move n library.  */
+static struct group *
+files_getgrent (void)
+{
+  enum nss_status status;
+  int buflen = 256;
+  char *buffer = NULL;
+  struct group *resultbuf = malloc (sizeof (struct group));
+
+  if (buffer == NULL)
+    buffer = malloc (buflen);
+
+  while ((status = files_getgrent_r (resultbuf, buffer, buflen, &errno))
+         == NSS_STATUS_TRYAGAIN && errno == ERANGE)
+    {
+      errno = 0;
+      buflen += 256;
+      buffer = realloc (buffer, buflen);
+    }
+  if (status == NSS_STATUS_SUCCESS)
+    return resultbuf;
+  else
+    return NULL;
+}
+
+struct group_list {
+  struct group *grp;
+  struct group_list *next;
+};
+
+static int
+sort_group_file (void)
+{
+  struct group *grp;
+  struct group_list *ptr = NULL;
+  int retval = 0;
+
+  while ((grp = files_getgrent ()))
+    {
+      if (grp->gr_name[0] == '+' || grp->gr_name[0] == '-')
+	break;
+
+      if (ptr == NULL)
+	{
+	  ptr = malloc (sizeof (struct group_list));
+	  if (ptr == NULL)
+	    {
+	      fputs ("running out of memory!\n", stderr);
+	      exit (E_FAILURE);
+	    }
+
+	  ptr->grp = grp;
+	  ptr->next = NULL;
+	}
+      else
+	{
+	  struct group_list *next = ptr;
+
+	  while (next->next && next->grp->gr_gid < grp->gr_gid)
+	    next = next->next;
+
+	  if (next->next == NULL)
+	    {
+	      next->next = malloc (sizeof (struct group_list));
+
+	      if (next->next == NULL)
+		{
+		  fputs ("running out of memory!\n", stderr);
+		  exit (E_FAILURE);
+		}
+	      next->next->next = NULL;
+
+	      if (grp->gr_gid > next->grp->gr_gid)
+		next->next->grp = grp;
+	      else
+		{
+		  next->next->grp = next->grp;
+		  next->grp = grp;
+		}
+	    }
+	  else
+	    {
+	      struct group_list *tmp;
+	      tmp = malloc (sizeof (struct group_list));
+
+	      if (tmp == NULL)
+		{
+		  fputs ("running out of memory!\n", stderr);
+		  exit (E_FAILURE);
+		}
+	      tmp->next = next->next;
+	      next->next = tmp;
+	      tmp->grp = next->grp;
+	      next->grp = grp;
+	    }
+	}
+    }
+
+  const char *file_tmp = "/group.tmpXXXXXX";
+  char *group_tmp = alloca (strlen (files_etc_dir) + strlen (file_tmp) + 1);
+  char *group_orig = alloca (strlen (files_etc_dir) + 8);
+  char *group_old = alloca (strlen (files_etc_dir) + 12);
+  struct stat group_stat;
+  FILE *oldgf, *newgf;
+  int newgf_fd;
+  char *cp;
+
+
+  cp = stpcpy (group_tmp, files_etc_dir);
+  strcpy (cp, file_tmp);
+  cp = stpcpy (group_orig, files_etc_dir);
+  strcpy (cp, "/group");
+  cp = stpcpy (group_old, group_orig);
+  strcpy (cp, ".old");
+
+  if ((oldgf = fopen (group_orig, "r")) == NULL)
+    {
+      fprintf (stderr, _("Can't open %s: %m\n"), group_orig);
+      retval = -1;
+      goto error_group;
+    }
+  if (fstat (fileno (oldgf), &group_stat) < 0)
+    {
+      fprintf (stderr, _("Can't stat %s: %m\n"), group_orig);
+      fclose (oldgf);
+      retval = -1;
+      goto error_group;
+    }
+
+#ifdef WITH_SELINUX
+  security_context_t prev_context;
+  if (set_default_context (group_orig, &prev_context) < 0)
+    {
+      fclose (oldgf);
+      retval = -1;
+      goto error_group;
+    }
+#endif
+  /* Open a temp group file */
+  newgf_fd = mkstemp (group_tmp);
+#ifdef WITH_SELINUX
+  restore_default_context (prev_context);
+#endif
+  if (newgf_fd == -1)
+    {
+      fprintf (stderr, _("Can't create %s temp file: %m\n"),
+	       group_orig);
+      fclose (oldgf);
+      retval = -1;
+      goto error_group;
+    }
+  fchmod (newgf_fd, group_stat.st_mode);
+  fchown (newgf_fd, group_stat.st_uid, group_stat.st_gid);
+  if (copy_xattr (group_orig, group_tmp) != 0)
+    {
+      fclose (oldgf);
+      close (newgf_fd);
+      unlink (group_tmp);
+      retval = -1;
+      goto error_group;
+    }
+
+  newgf = fdopen (newgf_fd, "w+");
+  if (newgf == NULL)
+    {
+      fprintf (stderr, _("Can't open %s: %m\n"), group_tmp);
+      fclose (oldgf);
+      close (newgf_fd);
+      unlink (group_tmp);
+      retval = -1;
+      goto error_group;
+    }
+
+  while (ptr != NULL)
+    {
+      /* write the group entry to tmp file */
+      if (putgrent (ptr->grp, newgf) < 0)
+	goto write_error_group;
+      ptr = ptr->next;
+    }
+
+  /* Check if we have entries starting with +/- and copy
+     the rest of the group file without sorting it. */
+  if (grp != NULL)
+    {
+      /* write the group entry to tmp file */
+      if (putgrent (grp, newgf) < 0)
+	goto write_error_group;
+      while ((grp = files_getgrent ()))
+      /* write the group entry to tmp file */
+      if (putgrent (grp, newgf) < 0)
+	{
+	write_error_group:
+	  fprintf (stderr,
+		   _("Error while writing temporary group file: %m\n"));
+	  fclose (oldgf);
+	  fclose (newgf);
+	  retval = -1;
+	  goto error_group;
+	}
+    }
+
+  if (fclose (oldgf) != 0)
+    {
+      fprintf (stderr,
+	       _("Error while closing old group file: %m\n"));
+      fclose (newgf);
+      retval = -1;
+      goto error_group;
+    }
+  if (fclose (newgf) != 0)
+    {
+      fprintf (stderr,
+	       _("Error while closing temporary group file: %m\n"));
+      retval = -1;
+      goto error_group;
+    }
+  unlink (group_old);
+  link (group_orig, group_old);
+  rename (group_tmp, group_orig);
+ error_group:
+  unlink (group_tmp);
+
+  return retval;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -528,5 +755,8 @@ main (int argc, char *argv[])
       return E_PWDBUSY;
     }
 
-  return loop_over_group_file (quiet);
+  if (sort)
+    return sort_group_file ();
+  else
+    return loop_over_group_file (quiet);
 }
