@@ -50,6 +50,10 @@
 #define _(String) gettext (String)
 #endif
 
+#ifndef MIN
+#define MIN(a,b) ((a)<(b)?(a):(b))
+#endif
+
 static void
 print_usage (FILE *stream, const char *program)
 {
@@ -78,6 +82,7 @@ main (int argc, char *argv[])
   char *program;
   char *cp, *shell;
   int l_flag = 0;
+  int ngroups_max = MIN (sysconf (_SC_NGROUPS_MAX), INT_MAX);
 
   setlocale(LC_ALL, "");
   bindtextdomain(PACKAGE, LOCALEDIR);
@@ -149,7 +154,7 @@ main (int argc, char *argv[])
       gid_t gid;
       gid_t egid;
       int is_member = 0;
-      int ngroups, i;
+      int ngroups, ngroups_allocated, i;
       gid_t *grouplist;
       char *utf8_arg = locale_to_utf8 (argv[0]);
 
@@ -199,17 +204,21 @@ main (int argc, char *argv[])
 	 enough memory for one additional group.  */
       ngroups = getgroups (0, 0);
       if (ngroups == -1)
-	ngroups = NGROUPS_MAX;
-      if( (ngroups+1) <= 0)
-        return E_FAILURE;
-      grouplist = malloc ((ngroups + 1) * sizeof (gid_t *));
+	ngroups = ngroups_max;
+      if ((ngroups+1) <= 0)
+	  /* Overflow, don't allocate more. */
+	  ngroups_allocated = ngroups;
+      else
+	ngroups_allocated = ngroups + 1;
+
+      grouplist = malloc (ngroups_allocated * sizeof (gid_t *));
       if (grouplist == NULL)
 	{
 	  fputs ("running out of memory!\n", stderr);
 	  return E_FAILURE;
 	}
 
-      if ((ngroups = getgroups (ngroups, (grouplist))) < 0)
+      if ((ngroups = getgroups (ngroups, grouplist)) < 0)
 	{
 	  fprintf (stderr, _("%s: calling getgroups failed: %s\n"),
 		   program, strerror (errno));
@@ -239,14 +248,16 @@ main (int argc, char *argv[])
 	grouplist[ngroups++] = egid;
 
 #else
-
       /* Add new gid to supp. list if it does not exist yet.  */
       for (i = 0; i < ngroups; i++)
 	if (grp->gr_gid == grouplist[i])
 	  break;
 
-      if (i == ngroups)
+      if (i == ngroups && ngroups < ngroups_allocated && ngroups < ngroups_max)
 	grouplist[ngroups++] = grp->gr_gid;
+      else
+	fprintf (stderr, _("%s: too many groups, not added.\n"),
+		 program);
 #endif
 
       if (setgroups (ngroups, (grouplist)) < 0)
