@@ -108,7 +108,7 @@ MySLPSrvURLCallback (SLPHandle hslp __attribute__((unused)),
 static int
 query_slp (HANDLE char **hostp, char **portp)
 {
-  struct slpcb callbackres;
+  struct slpcb callbackres = {NULL, 0};
   SLPError err;
   SLPHandle hslp;
 
@@ -123,7 +123,7 @@ query_slp (HANDLE char **hostp, char **portp)
       return err;
     }
 
-  err = SLPFindSrvs (hslp, "rpasswd.thkukuk", 0, 0,
+  err = SLPFindSrvs (hslp, "rpasswdd", 0, 0,
 		     MySLPSrvURLCallback, &callbackres);
 
   /* err may contain an error code that occurred as the slp library
