@@ -227,55 +227,6 @@ remove_gr_mem (const char *name, char **gr_mem)
   return groups;
 }
 
-
-#ifdef USE_LDAP
-static char *
-get_ldap_password (const char *binddn)
-{
-  /* A user tries to add an account stored in a LDAP database and
-     knows the Manager dn, now we need the password from him.  */
-  ldap_session_t *session = create_ldap_session (LDAP_PATH_CONF);
-  char *cp;
-
-  if (session == NULL)
-    return NULL;
-
-  cp = getpass (_("Enter LDAP Password:"));
-
-  if (open_ldap_session (session) != 0)
-    return NULL;
-
-  if (ldap_authentication (session, NULL, binddn, cp) != 0)
-    return NULL;
-
-  close_ldap_session (session);
-
-  return cp;
-}
-
-static char *
-get_caller_dn (void)
-{
-  /* A user tries to add an account stored in a LDAP database and
-     we need a DN, generate one for his account.  */
-  ldap_session_t *session = create_ldap_session (LDAP_PATH_CONF);
-  struct passwd *pw;
-  char *cp;
-
-  if (session == NULL)
-    return NULL;
-
-  pw = getpwuid (getuid ());
-
-  cp = convert_user_to_dn (session, strdupa (pw->pw_name));
-
-  close_ldap_session (session);
-
-  return cp;
-}
-#endif /* USE_LDAP */
-
-
 static int
 remove_from_secondary_groups (user_t *pw_data, int have_extrapath)
 {
